@@ -58,6 +58,7 @@ DEFINE_TEST(test_archive_write_add_filter_by_name_lzma)
 DEFINE_TEST(test_archive_write_add_filter_by_name_lzop)
 DEFINE_TEST(test_archive_write_add_filter_by_name_uuencode)
 DEFINE_TEST(test_archive_write_add_filter_by_name_xz)
+DEFINE_TEST(test_archive_write_add_filter_by_name_zstd)
 DEFINE_TEST(test_archive_write_set_filter_option)
 DEFINE_TEST(test_archive_write_set_format_by_name_7zip)
 DEFINE_TEST(test_archive_write_set_format_by_name_ar)
@@ -115,7 +116,6 @@ DEFINE_TEST(test_compat_lzip)
 DEFINE_TEST(test_compat_lzma)
 DEFINE_TEST(test_compat_lzop)
 DEFINE_TEST(test_compat_mac)
-DEFINE_TEST(test_compat_pax_libarchive_2x)
 DEFINE_TEST(test_compat_perl_archive_tar)
 DEFINE_TEST(test_compat_plexus_archiver_tar)
 DEFINE_TEST(test_compat_solaris_pax_sparse)
@@ -133,6 +133,8 @@ DEFINE_TEST(test_compat_zip_4)
 DEFINE_TEST(test_compat_zip_5)
 DEFINE_TEST(test_compat_zip_6)
 DEFINE_TEST(test_compat_zip_7)
+DEFINE_TEST(test_compat_zip_8)
+DEFINE_TEST(test_compat_zstd)
 DEFINE_TEST(test_empty_write)
 DEFINE_TEST(test_entry)
 DEFINE_TEST(test_entry_strmode)
@@ -268,8 +270,48 @@ DEFINE_TEST(test_read_format_mtree_nomagic_v1_form)
 DEFINE_TEST(test_read_format_mtree_nomagic_v2_form)
 DEFINE_TEST(test_read_format_mtree_nomagic_v2_netbsd_form)
 DEFINE_TEST(test_read_format_mtree_nonexistent_contents_file)
+DEFINE_TEST(test_read_format_mtree_noprint)
 DEFINE_TEST(test_read_format_mtree_crash747)
 DEFINE_TEST(test_read_format_pax_bz2)
+DEFINE_TEST(test_read_format_rar5_set_format)
+DEFINE_TEST(test_read_format_rar5_stored)
+DEFINE_TEST(test_read_format_rar5_compressed)
+DEFINE_TEST(test_read_format_rar5_multiple_files)
+DEFINE_TEST(test_read_format_rar5_multiple_files_solid)
+DEFINE_TEST(test_read_format_rar5_multiarchive_skip_all)
+DEFINE_TEST(test_read_format_rar5_multiarchive_skip_all_but_first)
+DEFINE_TEST(test_read_format_rar5_multiarchive_skip_all_but_second)
+DEFINE_TEST(test_read_format_rar5_blake2)
+DEFINE_TEST(test_read_format_rar5_arm_filter)
+DEFINE_TEST(test_read_format_rar5_stored_skip_all)
+DEFINE_TEST(test_read_format_rar5_stored_skip_in_part)
+DEFINE_TEST(test_read_format_rar5_stored_skip_all_but_first)
+DEFINE_TEST(test_read_format_rar5_stored_skip_all_in_part)
+DEFINE_TEST(test_read_format_rar5_multiarchive_solid_extr_all)
+DEFINE_TEST(test_read_format_rar5_multiarchive_solid_skip_all)
+DEFINE_TEST(test_read_format_rar5_multiarchive_solid_skip_all_but_first)
+DEFINE_TEST(test_read_format_rar5_multiarchive_solid_skip_all_but_scnd)
+DEFINE_TEST(test_read_format_rar5_multiarchive_solid_skip_all_but_third)
+DEFINE_TEST(test_read_format_rar5_multiarchive_solid_skip_all_but_last)
+DEFINE_TEST(test_read_format_rar5_solid_skip_all)
+DEFINE_TEST(test_read_format_rar5_solid_skip_all_but_first)
+DEFINE_TEST(test_read_format_rar5_solid_skip_all_but_second)
+DEFINE_TEST(test_read_format_rar5_solid_skip_all_but_last)
+DEFINE_TEST(test_read_format_rar5_extract_win32)
+DEFINE_TEST(test_read_format_rar5_block_by_block)
+DEFINE_TEST(test_read_format_rar5_owner)
+DEFINE_TEST(test_read_format_rar5_symlink)
+DEFINE_TEST(test_read_format_rar5_hardlink)
+DEFINE_TEST(test_read_format_rar5_extra_field_version)
+DEFINE_TEST(test_read_format_rar5_readtables_overflow)
+DEFINE_TEST(test_read_format_rar5_leftshift1)
+DEFINE_TEST(test_read_format_rar5_leftshift2)
+DEFINE_TEST(test_read_format_rar5_truncated_huff)
+DEFINE_TEST(test_read_format_rar5_invalid_dict_reference)
+DEFINE_TEST(test_read_format_rar5_distance_overflow)
+DEFINE_TEST(test_read_format_rar5_nonempty_dir_stream)
+DEFINE_TEST(test_read_format_rar5_fileattr)
+DEFINE_TEST(test_read_format_rar_set_format)
 DEFINE_TEST(test_read_format_rar_basic)
 DEFINE_TEST(test_read_format_rar_subblock)
 DEFINE_TEST(test_read_format_rar_noeof)
@@ -289,6 +331,7 @@ DEFINE_TEST(test_read_format_rar_multivolume_stored_file_skip)
 DEFINE_TEST(test_read_format_rar_multivolume_seek_data)
 DEFINE_TEST(test_read_format_rar_multivolume_seek_multiple_files)
 DEFINE_TEST(test_read_format_rar_multivolume_uncompressed_files)
+DEFINE_TEST(test_read_format_rar_ppmd_use_after_free)
 DEFINE_TEST(test_read_format_rar_encryption_data)
 DEFINE_TEST(test_read_format_rar_encryption_header)
 DEFINE_TEST(test_read_format_rar_encryption_partially)
@@ -298,6 +341,7 @@ DEFINE_TEST(test_read_format_tar)
 DEFINE_TEST(test_read_format_tar_concatenated)
 DEFINE_TEST(test_read_format_tar_empty_filename)
 DEFINE_TEST(test_read_format_tar_empty_pax)
+DEFINE_TEST(test_read_format_tar_empty_with_gnulabel)
 DEFINE_TEST(test_read_format_tar_filename)
 DEFINE_TEST(test_read_format_tbz)
 DEFINE_TEST(test_read_format_tgz)
@@ -307,11 +351,31 @@ DEFINE_TEST(test_read_format_tz)
 DEFINE_TEST(test_read_format_ustar_filename)
 DEFINE_TEST(test_read_format_warc)
 DEFINE_TEST(test_read_format_xar)
+DEFINE_TEST(test_read_format_zip_utf8_paths)
 DEFINE_TEST(test_read_format_zip)
+DEFINE_TEST(test_read_format_zip_ppmd_one_file)
+DEFINE_TEST(test_read_format_zip_ppmd_one_file_blockread)
+DEFINE_TEST(test_read_format_zip_ppmd_multi)
+DEFINE_TEST(test_read_format_zip_ppmd_multi_blockread)
+DEFINE_TEST(test_read_format_zip_lzma_one_file)
+DEFINE_TEST(test_read_format_zip_lzma_one_file_blockread)
+DEFINE_TEST(test_read_format_zip_lzma_multi)
+DEFINE_TEST(test_read_format_zip_lzma_multi_blockread)
+DEFINE_TEST(test_read_format_zip_bzip2_one_file)
+DEFINE_TEST(test_read_format_zip_bzip2_one_file_blockread)
+DEFINE_TEST(test_read_format_zip_bzip2_multi)
+DEFINE_TEST(test_read_format_zip_bzip2_multi_blockread)
+DEFINE_TEST(test_read_format_zip_xz_multi)
+DEFINE_TEST(test_read_format_zip_xz_multi_blockread)
+DEFINE_TEST(test_read_format_zip_ppmd8_crash_1)
+DEFINE_TEST(test_read_format_zip_bz2_hang_on_invalid)
+DEFINE_TEST(test_read_format_zip_ppmd8_crash_2)
+DEFINE_TEST(test_read_format_zip_lzma_alone_leak)
 DEFINE_TEST(test_read_format_zip_comment_stored)
 DEFINE_TEST(test_read_format_zip_encryption_data)
 DEFINE_TEST(test_read_format_zip_encryption_header)
 DEFINE_TEST(test_read_format_zip_encryption_partially)
+DEFINE_TEST(test_read_format_zip_extra_padding)
 DEFINE_TEST(test_read_format_zip_filename_CP932_eucJP)
 DEFINE_TEST(test_read_format_zip_filename_CP932_UTF8)
 DEFINE_TEST(test_read_format_zip_filename_UTF8_eucJP)
@@ -347,6 +411,7 @@ DEFINE_TEST(test_read_format_zip_winzip_aes128)
 DEFINE_TEST(test_read_format_zip_winzip_aes256)
 DEFINE_TEST(test_read_format_zip_winzip_aes256_stored)
 DEFINE_TEST(test_read_format_zip_winzip_aes256_large)
+DEFINE_TEST(test_read_format_zip_with_invalid_traditional_eocd)
 DEFINE_TEST(test_read_format_zip_zip64a)
 DEFINE_TEST(test_read_format_zip_zip64b)
 DEFINE_TEST(test_read_large)
@@ -419,6 +484,7 @@ DEFINE_TEST(test_write_filter_lzop)
 DEFINE_TEST(test_write_filter_program)
 DEFINE_TEST(test_write_filter_uuencode)
 DEFINE_TEST(test_write_filter_xz)
+DEFINE_TEST(test_write_filter_zstd)
 DEFINE_TEST(test_write_format_7zip)
 DEFINE_TEST(test_write_format_7zip_basic_bzip2)
 DEFINE_TEST(test_write_format_7zip_basic_copy)
@@ -485,6 +551,7 @@ DEFINE_TEST(test_write_read_format_zip)
 DEFINE_TEST(test_write_read_format_zip_improved_streaming)
 DEFINE_TEST(test_write_read_format_zip64)
 DEFINE_TEST(test_write_read_format_zip64_improved_streaming)
+DEFINE_TEST(test_xattr_platform)
 DEFINE_TEST(test_zip_filename_encoding_UTF8)
 DEFINE_TEST(test_zip_filename_encoding_KOI8R)
 DEFINE_TEST(test_zip_filename_encoding_ru_RU_CP1251)
