@@ -58,6 +58,14 @@ test_fuzz(const struct files *filesets)
 	size_t blk_size;
 	int64_t blk_offset;
 	int n;
+	const char *skip_fuzz_tests;
+
+	skip_fuzz_tests = getenv("SKIP_TEST_FUZZ");
+	if (skip_fuzz_tests != NULL) {
+		skipping("Skipping fuzz tests due to SKIP_TEST_FUZZ "
+		    "environment variable");
+		return;
+	}
 
 	for (n = 0; filesets[n].names != NULL; ++n) {
 		const size_t buffsize = 30000000;
@@ -110,8 +118,9 @@ test_fuzz(const struct files *filesets)
 		} else {
 			for (i = 0; filesets[n].names[i] != NULL; ++i)
 			{
+				char *newraw;
 				tmp = slurpfile(&size, filesets[n].names[i]);
-				char *newraw = realloc(rawimage, oldsize + size);
+				newraw = realloc(rawimage, oldsize + size);
 				if (!assert(newraw != NULL))
 				{
 					free(rawimage);
@@ -411,6 +420,12 @@ DEFINE_TEST(test_fuzz_tar)
 		"test_compat_lzop_1.tar.lzo",
 		NULL
 	};
+#endif
+#if HAVE_ZSTD_H && HAVE_LIBZSTD
+	static const char *fileset10[] = {
+		"test_compat_zstd_1.tar.zst",
+		NULL
+	};
 #endif
 	static const struct files filesets[] = {
 		{0, fileset1}, /* Exercise bzip2 decompressor. */
@@ -424,6 +439,9 @@ DEFINE_TEST(test_fuzz_tar)
 		{0, fileset8},
 #if HAVE_LIBLZO2 && HAVE_LZO_LZO1X_H && HAVE_LZO_LZOCONF_H
 		{0, fileset9}, /* Exercise lzo decompressor. */
+#endif
+#if HAVE_ZSTD_H && HAVE_LIBZSTD
+		{0, fileset10}, /* Exercise zstd decompressor. */
 #endif
 		{1, NULL}
 	};
