@@ -57,7 +57,7 @@ __FBSDID("$FreeBSD$");
 
 /*
  * NOTE: On FreeBSD and Solaris, this test needs ZFS.
- * You may should perfom this test as
+ * You may perform this test as
  * 'TMPDIR=<a directory on the ZFS> libarchive_test'.
  */
 
@@ -68,6 +68,14 @@ struct sparse {
 
 static void create_sparse_file(const char *, const struct sparse *);
 
+#if defined(__APPLE__)
+/* On APFS holes need to be at least 4096x4097 bytes */
+#define MIN_HOLE 16781312
+#else
+/* Elsewhere we work with 4096*10 bytes */
+#define MIN_HOLE 409600
+#endif
+
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #include <winioctl.h>
 /*
@@ -118,13 +126,26 @@ create_sparse_file(const char *path, const struct sparse *s)
 	assert(handle != INVALID_HANDLE_VALUE);
 	assert(DeviceIoControl(handle, FSCTL_SET_SPARSE, NULL, 0,
 	    NULL, 0, &dmy, NULL) != 0);
+
+	size_t offsetSoFar = 0;
+
 	while (s->type != END) {
 		if (s->type == HOLE) {
-			LARGE_INTEGER distance;
+			LARGE_INTEGER fileOffset, beyondOffset, distanceToMove;
+			fileOffset.QuadPart = offsetSoFar;
+			beyondOffset.QuadPart = offsetSoFar + s->size;
+			distanceToMove.QuadPart = s->size;
 
-			distance.QuadPart = s->size;
-			assert(SetFilePointerEx(handle, distance,
-			    NULL, FILE_CURRENT) != 0);
+			FILE_ZERO_DATA_INFORMATION zeroInformation;
+			zeroInformation.FileOffset = fileOffset;
+			zeroInformation.BeyondFinalZero = beyondOffset;
+
+			DWORD bytesReturned;
+			assert(SetFilePointerEx(handle, distanceToMove,
+				NULL, FILE_CURRENT) != 0);
+			assert(SetEndOfFile(handle) != 0);
+			assert(DeviceIoControl(handle, FSCTL_SET_ZERO_DATA, &zeroInformation,
+				sizeof(FILE_ZERO_DATA_INFORMATION), NULL, 0, &bytesReturned, NULL) != 0);
 		} else {
 			DWORD w, wr;
 			size_t size;
@@ -139,6 +160,7 @@ create_sparse_file(const char *path, const struct sparse *s)
 				size -= wr;
 			}
 		}
+		offsetSoFar += s->size;
 		s++;
 	}
 	assertEqualInt(CloseHandle(handle), 1);
@@ -146,28 +168,14 @@ create_sparse_file(const char *path, const struct sparse *s)
 
 #else
 
-#if defined(_PC_MIN_HOLE_SIZE)
-
-/*
- * FreeBSD and Solaris can detect 'hole' of a sparse file
- * through lseek(HOLE) on ZFS. (UFS does not support yet)
- */
-
-static int
-is_sparse_supported(const char *path)
-{
-	return (pathconf(path, _PC_MIN_HOLE_SIZE) > 0);
-}
-
-#elif defined(__linux__)&& defined(HAVE_LINUX_FIEMAP_H)
-
+#if defined(HAVE_LINUX_FIEMAP_H)
 /*
  * FIEMAP, which can detect 'hole' of a sparse file, has
  * been supported from 2.6.28
  */
 
 static int
-is_sparse_supported(const char *path)
+is_sparse_supported_fiemap(const char *path)
 {
 	const struct sparse sparse_file[] = {
  		/* This hole size is too small to create a sparse
@@ -198,7 +206,58 @@ is_sparse_supported(const char *path)
 	return (r >= 0);
 }
 
-#else
+#if !defined(SEEK_HOLE) || !defined(SEEK_DATA)
+static int
+is_sparse_supported(const char *path)
+{
+	return is_sparse_supported_fiemap(path);
+}
+#endif
+#endif
+
+#if defined(_PC_MIN_HOLE_SIZE)
+
+/*
+ * FreeBSD and Solaris can detect 'hole' of a sparse file
+ * through lseek(HOLE) on ZFS. (UFS does not support yet)
+ */
+
+static int
+is_sparse_supported(const char *path)
+{
+	return (pathconf(path, _PC_MIN_HOLE_SIZE) > 0);
+}
+
+#elif defined(SEEK_HOLE) && defined(SEEK_DATA)
+
+static int
+is_sparse_supported(const char *path)
+{
+	const struct sparse sparse_file[] = {
+ 		/* This hole size is too small to create a sparse
+		 * files for almost filesystem. */
+		{ HOLE,	 1024 }, { DATA, 10240 },
+		{ END,	0 }
+	};
+	int fd, r;
+	const char *testfile = "can_sparse";
+
+	(void)path; /* UNUSED */
+	create_sparse_file(testfile, sparse_file);
+	fd = open(testfile,  O_RDWR);
+	if (fd < 0)
+		return (0);
+	r = lseek(fd, 0, SEEK_HOLE);
+	close(fd);
+	unlink(testfile);
+#if defined(HAVE_LINUX_FIEMAP_H)
+	if (r < 0)
+		return (is_sparse_supported_fiemap(path));
+#endif
+	return (r >= 0);
+}
+
+#elif !defined(HAVE_LINUX_FIEMAP_H)
 
 /*
  * Other system may do not have the API such as lseek(HOLE),
@@ -371,6 +430,7 @@ verify_sparse_file(struct archive *a, const char *path,
 	assert(sparse->type == END);
 	assertEqualInt(expected_offset, archive_entry_size(ae));
 
+	failure(path);
 	assertEqualInt(holes_seen, expected_holes);
 
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
@@ -406,6 +466,7 @@ verify_sparse_file2(struct archive *a, const char *path,
 	/* Verify the number of holes only, not its offset nor its
 	 * length because those alignments are deeply dependence on
 	 * its filesystem. */ 
+	failure(path);
 	assertEqualInt(blocks, archive_entry_sparse_count(ae));
 	archive_entry_free(ae);
 }
@@ -438,6 +499,7 @@ DEFINE_TEST(test_sparse_basic)
 {
 	char *cwd;
 	struct archive *a;
+	const char *skip_sparse_tests;
 	/*
 	 * The alignment of the hole of sparse files deeply depends
 	 * on filesystem. In my experience, sparse_file2 test with
@@ -447,38 +509,43 @@ DEFINE_TEST(test_sparse_basic)
 	 * on all platform.
 	 */
 	const struct sparse sparse_file0[] = {
-		{ DATA,	 1024 }, { HOLE,   2048000 },
-		{ DATA,	 2048 }, { HOLE,   2048000 },
-		{ DATA,	 4096 }, { HOLE,  20480000 },
-		{ DATA,	 8192 }, { HOLE, 204800000 },
+		// 0             // 1024
+		{ DATA,	 1024 }, { HOLE,   MIN_HOLE + 1638400 },
+		// 2049024       // 2051072
+		{ DATA,	 2048 }, { HOLE,   MIN_HOLE + 1638400 },
+		// 4099072       // 4103168
+		{ DATA,	 4096 }, { HOLE,  MIN_HOLE + 20070400 },
+		// 24583168      // 24591360
+		{ DATA,	 8192 }, { HOLE, MIN_HOLE + 204390400 },
+		// 229391360     // 229391361
 		{ DATA,     1 }, { END,	0 }
 	};
 	const struct sparse sparse_file1[] = {
-		{ HOLE,	409600 }, { DATA, 1 },
-		{ HOLE,	409600 }, { DATA, 1 },
-		{ HOLE,	409600 }, { END,  0 }
+		{ HOLE,	MIN_HOLE }, { DATA, 1 },
+		{ HOLE,	MIN_HOLE }, { DATA, 1 },
+		{ HOLE, MIN_HOLE }, { END,  0 }
 	};
 	const struct sparse sparse_file2[] = {
-		{ HOLE,	409600 * 1 }, { DATA, 1024 },
-		{ HOLE,	409600 * 2 }, { DATA, 1024 },
-		{ HOLE,	409600 * 3 }, { DATA, 1024 },
-		{ HOLE,	409600 * 4 }, { DATA, 1024 },
-		{ HOLE,	409600 * 5 }, { DATA, 1024 },
-		{ HOLE,	409600 * 6 }, { DATA, 1024 },
-		{ HOLE,	409600 * 7 }, { DATA, 1024 },
-		{ HOLE,	409600 * 8 }, { DATA, 1024 },
-		{ HOLE,	409600 * 9 }, { DATA, 1024 },
-		{ HOLE,	409600 * 10}, { DATA, 1024 },/* 10 */
-		{ HOLE,	409600 * 1 }, { DATA, 1024 * 1 },
-		{ HOLE,	409600 * 2 }, { DATA, 1024 * 2 },
-		{ HOLE,	409600 * 3 }, { DATA, 1024 * 3 },
-		{ HOLE,	409600 * 4 }, { DATA, 1024 * 4 },
-		{ HOLE,	409600 * 5 }, { DATA, 1024 * 5 },
-		{ HOLE,	409600 * 6 }, { DATA, 1024 * 6 },
-		{ HOLE,	409600 * 7 }, { DATA, 1024 * 7 },
-		{ HOLE,	409600 * 8 }, { DATA, 1024 * 8 },
-		{ HOLE,	409600 * 9 }, { DATA, 1024 * 9 },
-		{ HOLE,	409600 * 10}, { DATA, 1024 * 10},/* 20 */
+		{ HOLE,	MIN_HOLE }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 1 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 2 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 3 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 4 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 5 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 6 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 7 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 8 }, { DATA, 1024 },
+		{ HOLE,	MIN_HOLE + 409600 * 9}, { DATA, 1024 },/* 10 */
+		{ HOLE,	MIN_HOLE }, { DATA, 1024 * 1 },
+		{ HOLE,	MIN_HOLE + 409600 * 1 }, { DATA, 1024 * 2 },
+		{ HOLE,	MIN_HOLE + 409600 * 2 }, { DATA, 1024 * 3 },
+		{ HOLE,	MIN_HOLE + 409600 * 3 }, { DATA, 1024 * 4 },
+		{ HOLE,	MIN_HOLE + 409600 * 4 }, { DATA, 1024 * 5 },
+		{ HOLE,	MIN_HOLE + 409600 * 5 }, { DATA, 1024 * 6 },
+		{ HOLE,	MIN_HOLE + 409600 * 6 }, { DATA, 1024 * 7 },
+		{ HOLE,	MIN_HOLE + 409600 * 7 }, { DATA, 1024 * 8 },
+		{ HOLE,	MIN_HOLE + 409600 * 8 }, { DATA, 1024 * 9 },
+		{ HOLE,	MIN_HOLE + 409600 * 9}, { DATA, 1024 * 10},/* 20 */
 		{ END,	0 }
 	};
 	const struct sparse sparse_file3[] = {
@@ -495,6 +562,13 @@ DEFINE_TEST(test_sparse_basic)
 	 */
 	test_sparse_whole_file_data();
 
+	skip_sparse_tests = getenv("SKIP_TEST_SPARSE");
+	if (skip_sparse_tests != NULL) {
+		skipping("Skipping sparse tests due to SKIP_TEST_SPARSE "
+		    "environment variable");
+		return;
+	}
+
 	/* Check if the filesystem where CWD on can
 	 * report the number of the holes of a sparse file. */
 #ifdef PATH_MAX
@@ -541,10 +615,19 @@ DEFINE_TEST(test_fully_sparse_files)
 {
 	char *cwd;
 	struct archive *a;
+	const char *skip_sparse_tests;
 
 	const struct sparse sparse_file[] = {
-		{ HOLE, 409600 }, { END, 0 }
+		{ HOLE, MIN_HOLE }, { END, 0 }
 	};
+
+	skip_sparse_tests = getenv("SKIP_TEST_SPARSE");
+	if (skip_sparse_tests != NULL) {
+		skipping("Skipping sparse tests due to SKIP_TEST_SPARSE "
+		    "environment variable");
+		return;
+	}
+
 	/* Check if the filesystem where CWD on can
 	 * report the number of the holes of a sparse file. */
 #ifdef PATH_MAX
